@@ -11,6 +11,28 @@ import os
 import pathlib
 
 
+class Slide(bytes):
+    """
+    Helper class that extends bytes. It's expected the loaded bytes are a png image.
+    The class displays the image as representation when used in Jupyter Notebooks without any further coding requirement
+    for the user.
+
+    As additional convince functions, the bytes can also be easily saved to file or converted to a PIL image.
+    """
+    def save(self, file):
+        with open(file, 'wb') as img:
+            img.write(self)
+
+    def to_pil_image(self):
+        stream = BytesIO(self)
+        pil_image = Image.open(stream)
+        pil_image.load()
+        return pil_image
+
+    def _repr_png_(self):
+        return self
+
+
 class SlideGenerator(object):
     """Molecule Overview Slide Generator
 
@@ -28,7 +50,7 @@ class SlideGenerator(object):
     LINUX_TRUETYPE_FONTS_FOLDER = '/usr/share/fonts/truetype'
 
     def __init__(self, mols_per_row=7, rows=3, font_size=16, font='Calibrib', number_of_properties=4, bond_length=20,
-                 bond_width=2, slide_width=1440, slide_height=607, dpi = (120,120)):
+                 bond_width=2, slide_width=1440, slide_height=607, dpi=(120,120), comic_mode=False):
         """
         The parameters defined are used to calculate the size that each molecule and the properties part can take up.
 
@@ -39,14 +61,14 @@ class SlideGenerator(object):
         some manual experiments to figure out optimal settings.
 
         'font' must either be a font name of a installed font or the full path to a true type font file (.ttf).
-        For Windows the named fonts must be in 'C:/Windows/Fonts/' directory and for Linux in '/usr/share/fonts/truetype'.
-        (Linux font folder is for Ubuntu. Don't know if it is the same for other distros).
+        For Windows the named fonts must be in 'C:/Windows/Fonts/' directory and for Linux in
+        '/usr/share/fonts/truetype'. (Linux font folder is for Ubuntu. Don't know if it is the same for other distros).
         The font is used for the atom labels and the properties.
 
-        'number_of_properties' defines how many additional data you want to show below the molecules. Each property takes
-        up 1 line of text. Example: pIC50: 6.2. This is meant for very small amounts of text/data. If text is too long,
-        it is simply truncated automatically due to image width limit. It's up to the caller to adjust the data if hard
-        truncation isn't acceptable. The approximate number of characters per line can be deduced from the instance
+        'number_of_properties' defines how many additional data you want to show below the molecules. Each property
+        takes up 1 line of text. Example: pIC50: 6.2. This is meant for very small amounts of text/data. If text is too
+        long, it is simply truncated automatically due to image width limit. It's up to the caller to adjust the data if
+        hard truncation isn't acceptable. The approximate number of characters per line can be deduced from the instance
         attribute 'num_chars_per_line'. Exact amount depends on the exact text.
 
         :param mols_per_row: number of molecules per row. default:7
@@ -54,12 +76,17 @@ class SlideGenerator(object):
         :param font_size: size of the properties font. default:16
         :param font: name of the font to use or path to a .ttf font file
         :param number_of_properties: number of properties to display below the molecule
+        :param bond_length: maximum bond length. if needed will be scaled down to fit image
+        :param bond_width: width of bonds
+        :param slide_width: width of the generated image
+        :param slide_height: height of the generated image
+        :param dpi: resolution of the generated image. A 2-tuple, 1 for each dimension
         """
 
         root, ext = os.path.splitext(font)
         platform_system = platform.system()
         if ext != '' and ext != '.ttf':
-            raise ValueError("'font' must be a path to an existing ttf file. However {} was provided as font.".format(font))
+            raise ValueError(f"'font' must be a path to an existing ttf file. However {font} was provided as font.")
         elif ext == '.ttf':
             self.font_path = font
         elif platform_system == 'Windows':
@@ -73,8 +100,8 @@ class SlideGenerator(object):
                     self.font_path = os.path.join(root, font_file_name)
                     break
         else:
-            raise ValueError ("'font' was set to name {} but the location of the font folder is unknown for platform "
-                              "{}. Please use full path to font file.".format(font, platform_system))
+            raise ValueError(f"'font' was set to name {font} but the location of the font folder is unknown for "
+                             f"platform {platform_system}. Please use full path to font file.")
 
         p = pathlib.Path(self.font_path)
         if not p.is_file():
@@ -99,6 +126,7 @@ class SlideGenerator(object):
         avg_char_width = self.font.getsize('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789-, ')[0] / 64
         self.num_chars_per_line = self.image_width / avg_char_width
         self.bond_length = bond_length
+        self.comic_mode = comic_mode
 
     def generate_slide(self, mols, properties, out_path=None):
         """
@@ -124,12 +152,12 @@ class SlideGenerator(object):
         """
         Generates an image with all molecules and their properties below them.
 
-        Images is a list containing the molecules images as PIL images to use in the slide. They must be in same order as mols.
-        This allows each molecule image to be generated differently like substructure highlighting, annotations and so
-        forth. So many options they can't be all part of a methods argument.
+        Images is a list containing the molecules images as PIL images to use in the slide. They must be in same order
+        as mols. This allows each molecule image to be generated differently like substructure highlighting, annotations
+        and so forth. So many options they can't be all part of a methods' argument.
 
         :param mols: list of rdkit molecules
-        :param properties: list of list of TextProperty
+        :param properties: list of lists of TextProperty
         :param images: list of images to use for each molecule
         :param out_path: optional file to save generated images
         :return: bytes object containing the png image data say for display in a notebook
@@ -166,13 +194,13 @@ class SlideGenerator(object):
             raise ValueError('Number of molecules must match number of properties.')
 
         if len(mols) == self.max_mols:
-            slide = Image.new('RGBA', [self.slide_width, self.slide_height], (255, 255, 255, 0))
+            slide = Image.new('RGBA', (self.slide_width, self.slide_height), (255, 255, 255, 0))
         # less mols -> smaller image
         elif len(mols) < self.mols_per_row:
-            slide = Image.new('RGBA', [len(mols) * self.image_width, self.row_height], (255, 255, 255, 0))
+            slide = Image.new('RGBA', (len(mols) * self.image_width, self.row_height), (255, 255, 255, 0))
         else:
             num_rows = (len(mols) // self.mols_per_row) + 1
-            slide = Image.new('RGBA', [self.slide_width, self.row_height * num_rows], (255, 255, 255, 0))
+            slide = Image.new('RGBA', (self.slide_width, self.row_height * num_rows), (255, 255, 255, 0))
 
         png_info = PngInfo()
         png_info.add_text('numProperties', str(self.number_of_properties))
@@ -184,7 +212,7 @@ class SlideGenerator(object):
         if self.number_of_properties > 0:
             text_image = self._draw_text(properties[index])
 
-            combined = Image.new('RGBA', [self.image_width, self.row_height], (255, 255, 255, 0))
+            combined = Image.new('RGBA', (self.image_width, self.row_height), (255, 255, 255, 0))
             combined.paste(mol_image)
             combined.paste(text_image, (0, self.molecule_image_height))
         else:
@@ -222,8 +250,8 @@ class SlideGenerator(object):
 
         img_byte_arr = BytesIO()
         slide.save(img_byte_arr, format='PNG', dpi=self.dpi, pnginfo=png_info)
-        img_byte_arr = img_byte_arr.getvalue()
-        return img_byte_arr
+        # img_byte_arr = img_byte_arr.getvalue()
+        return Slide(img_byte_arr.getvalue())
 
     def _draw_text(self, mol_properties):
 
@@ -254,6 +282,7 @@ class SlideGenerator(object):
         opts.minFontSize = self.font_size
         opts.maxFontSize = self.font_size
         opts.fontFile = self.font_path
+        opts.comicMode = self.comic_mode
 
         drawer.DrawMolecule(mol)
         drawer.FinishDrawing()
@@ -264,7 +293,14 @@ class SlideGenerator(object):
 
     @staticmethod
     def _scale_bond_length(mol):
+        """
+        This helper function scales a molecules bond length so that all bonds have the RDKit default bond length of
+        1.5A. This is needed as input from sd-files can have very variable bond length and the output will then not
+        look very nice.
 
+        :param mol: the rdkit molecule
+        :return: molecule with bond length changed to 1.5
+        """
         default_bond_length = 1.5  # rdkit default bond length
 
         bonds = mol.GetBonds()
@@ -310,7 +346,7 @@ class TextProperty(object):
         :param name: display name of the property
         :param value: the value of the property
         :param show_name: if the name of the property should be displayed or not
-        :param color: color of the text for this property. Default is black.
+        :param color: color of the text for this property. Default is black. HEX or RGBA 4-tuple.
         """
         self.name = name
         self.value = value
